#!/usr/bin/python


# Convert CSV files into SQL tables and insert statements

import csv, sys

try:
	import win32file
except:
	win32file = None


def log(message):
	if not quiet:
		try:
			print message
		except UnicodeEncodeError:
			print message.encode("utf8")


def logError(message):
	try:
		sys.stderr.write(message + "\n")
	except UnicodeEncodeError:
		sys.stderr.write(message.encode("utf8"))


def getSQLHeader():
	return """
-- csv2sql conversion
-- http://jeff.mikels.cc
-- 
-- --------------------------------------------------------
-- 
"""



def makeCreateTable(h):
	global tableName
	retval = "\nDROP TABLE IF EXISTS `%s`;" % tableName
	retval += "\n-- HEADER ROW\n-- %s" % "//".join(h)
	retval += "\nCREATE TABLE `%s` (" % tableName
	retval += "\n  `_ID` MEDIUMINT NOT NULL AUTO_INCREMENT,"

	for v in h:
		retval += "\n  `%s` TEXT," % v.replace(' ','_').replace('_ID','_old_ID')

	retval += "\n  PRIMARY KEY (`_ID`)"
	retval += "\n);\n\n"

	return retval


def makeInsert(h,r):
	global tableName
	retval = ""
	retval += "\n-- DATA ROW\n-- %s" % " / ".join(r).replace('\n','\\n').replace('\r','\\r')
	if len(r) > len(h):
		retval += "\n-- SKIPPING THIS ROW: DATA HAS MORE ELEMENTS THAN HEADER.\n"
		return retval


	while len(r) < len(h):
		r.append("")


	retval += "\nINSERT INTO `%s` (" % tableName

	for v in h[:-1]:
		if v == '_ID':
			v = '_old_ID'
		retval += "`%s`," % v.replace(' ','_')

	v = h[-1]
	retval += "`%s`) VALUES (" % v.replace(' ','_')

	for v in r[:-1]:
		retval += "'%s'," % v.replace("'","''")

	v = r[-1]
	retval += "'%s');\n\n" % v.replace("'","''")
	return retval


def printUsage():
	print """---------------------------------------------
USAGE: csv2sql.py input [output] [tableName]

If output is not specified or is '-', sql statements will be sent to stdout."

If tableName is not specified, the table name will be automatically generated
from input filename or output filename.

	'contacts.sql' --> tablename = 'contacts'."""

def main(argv):
	global tableName

	if len(argv) <= 1:
		printUsage()
		return 1

	input = file(argv[1],'r')
	tableName = argv[1].replace('.csv','').replace(' ','_')

	if len(argv) == 2:
		output = sys.stdout
	else:
		if argv[2] == '-':
			output = sys.stdout
		else:
			output = file(argv[2],'w')
			tableName = argv[2].replace('.sql','').replace(' ','_')

	if len(argv) == 4:
		tableName = argv[3].replace(' ','_')

	c = csv.reader(input,dialect='excel')
	data = []

	for row in c:
		data.append(row)

	input.close()

	header = data[0]
	
	output.write(getSQLHeader())
	output.write(makeCreateTable(header))

	for row in data[1:]:
		#sql = "%s\n%s" % (sql,makeInsert(header, row))
		output.write(makeInsert(header, row))

	output.close()
	return 0

if __name__ == "__main__":
	sys.exit(main(sys.argv))
